import matplotlib.pyplot as plt
import numpy as np
import open3d as o3d

# pylint: disable=invalid-name,unused-import
from mpl_toolkits.mplot3d import Axes3D


def get_pts(pcd):
    points = np.asarray(pcd.points)
    X = []
    Y = []
    Z = []
    for pt in range(points.shape[0]):
        X.append(points[pt][0])
        Y.append(points[pt][1])
        Z.append(points[pt][2])
    return np.asarray(X), np.asarray(Y), np.asarray(Z)


def set_axes_equal(ax):
    # x_limits = ax.get_xlim3d()
    # y_limits = ax.get_ylim3d()
    # z_limits = ax.get_zlim3d()
    # x_range = abs(x_limits[1] - x_limits[0])
    # x_middle = np.mean(x_limits)
    # y_range = abs(y_limits[1] - y_limits[0])
    # y_middle = np.mean(y_limits)
    # z_range = abs(z_limits[1] - z_limits[0])
    # z_middle = np.mean(z_limits)
    # plot_radius = 0.5 * max([x_range, y_range, z_range])
    # ax.set_xlim3d([x_middle - plot_radius, x_middle + plot_radius])
    # ax.set_ylim3d([y_middle - plot_radius, y_middle + plot_radius])
    # ax.set_zlim3d([z_middle - plot_radius, z_middle + plot_radius])

    limit = 0.3
    ax.set_xlim3d([-limit, limit])
    ax.set_ylim3d([-limit, limit])
    ax.set_zlim3d([-limit, limit])


def plot_single_pcd(points, save_path, title=None):
    fig = plt.figure()
    ax = fig.add_subplot(111, projection="3d")
    ax.set_aspect("auto")
    pcd = o3d.geometry.PointCloud(o3d.utility.Vector3dVector(points))
    rotation_matrix = np.asarray(
        [[1, 0, 0, 0], [0, 0, -1, 0], [0, 1, 0, 0], [0, 0, 0, 1]]
    )
    pcd = pcd.transform(rotation_matrix)
    X, Y, Z = get_pts(pcd)
    t = Z
    ax.scatter(X, Y, Z, c=t, cmap="jet", marker="o", s=2, linewidths=0)
    ax.grid(False)
    ax.w_xaxis.set_pane_color((1.0, 1.0, 1.0, 1.0))
    ax.w_yaxis.set_pane_color((1.0, 1.0, 1.0, 1.0))
    ax.w_zaxis.set_pane_color((1.0, 1.0, 1.0, 1.0))
    set_axes_equal(ax)
    plt.axis("off")
    if title is not None:
        plt.title(title)
    plt.tight_layout()
    plt.savefig(save_path, bbox_inches="tight", format="png", dpi=300)
    plt.close()


def plot_batch_pcd(points, save_path, n_col=5):
    batch_size = points.shape[0]
    n_row = int(np.ceil(batch_size / n_col))
    fig, axes = plt.subplots(n_row, n_col, subplot_kw={"projection": "3d"})
    rotation_matrix = np.asarray(
        [[1, 0, 0, 0], [0, 0, -1, 0], [0, 1, 0, 0], [0, 0, 0, 1]]
    )

    for idx_row in range(n_row):
        for idx_col in range(n_col):
            axes[idx_row, idx_col].set_aspect("auto")
            pcd = o3d.geometry.PointCloud(
                o3d.utility.Vector3dVector(points[idx_row * n_col + idx_col])
            )
            pcd = pcd.transform(rotation_matrix)
            X, Y, Z = get_pts(pcd)
            t = Z
            axes[idx_row, idx_col].scatter(
                X, Y, Z, c=t, cmap="jet", marker="o", s=2, linewidths=0
            )
            axes[idx_row, idx_col].grid(False)
            axes[idx_row, idx_col].w_xaxis.set_pane_color((1.0, 1.0, 1.0, 1.0))
            axes[idx_row, idx_col].w_yaxis.set_pane_color((1.0, 1.0, 1.0, 1.0))
            axes[idx_row, idx_col].w_zaxis.set_pane_color((1.0, 1.0, 1.0, 1.0))
            set_axes_equal(axes[idx_row, idx_col])
            axes[idx_row, idx_col].axis("off")

    fig.tight_layout()
    fig.savefig(save_path, bbox_inches="tight", format="png", dpi=300)
    plt.close()
